@@ -6,52 +6,68 @@
  * Copyright 2002 Project Purple
  */
 
-//#include <stdint.h>
 #include <inttypes.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include "armor.h"
+#include "charfuncs.h"
+#include "cleankey.h"
+#include "cleanup.h"
+#include "config.h"
 #include "getcgi.h"
 #include "keydb.h"
 #include "keyindex.h"
+#include "log.h"
 #include "mem.h"
-#include "onak_conf.h"
+#include "onak-conf.h"
 #include "parsekey.h"
+#include "photoid.h"
 
 #define OP_UNKNOWN 0
 #define OP_GET     1
 #define OP_INDEX   2
 #define OP_VINDEX  3
-
-int putnextchar(void *ctx, unsigned char c)
-{
-        return putchar(c);
-}
+#define OP_PHOTO   4
 
 void find_keys(char *search, uint64_t keyid, bool ishex,
-		bool fingerprint, bool exact, bool verbose)
+		bool fingerprint, bool exact, bool verbose, bool mrhkp)
 {
 	struct openpgp_publickey *publickey = NULL;
 	int count = 0;
 
 	if (ishex) {
-		count = fetch_key(keyid, &publickey);
+		count = fetch_key(keyid, &publickey, false);
 	} else {
 		count = fetch_key_text(search, &publickey);
 	}
 	if (publickey != NULL) {
-		key_index(publickey, verbose, fingerprint, true);
+		if (mrhkp) {
+			printf("info:1:%d\n", count);
+			mrkey_index(publickey);
+		} else {
+			key_index(publickey, verbose, fingerprint, true);
+		}
 		free_publickey(publickey);
 	} else if (count == 0) {
-		puts("Key not found.");
+		if (mrhkp) {
+			puts("info:1:0");
+		} else {
+			puts("Key not found.");
+		}
 	} else {
-		printf("Found %d keys, but maximum number to return is %d.\n",
+		if (mrhkp) {
+			puts("info:1:0");
+		} else {
+			printf("Found %d keys, but maximum number to return"
+				" is %d.\n",
 				count,
 				config.maxkeys);
-		puts("Try again with a more specific search.");
+			puts("Try again with a more specific search.");
+		}
 	}
 }
 
@@ -60,9 +76,11 @@ int main(int argc, char *argv[])
 	char **params = NULL;
 	int op = OP_UNKNOWN;
 	int i;
+	int indx = 0;
 	bool fingerprint = false;
 	bool exact = false;
 	bool ishex = false;
+	bool mrhkp = false;
 	uint64_t keyid = 0;
 	char *search = NULL;
 	char *end = NULL;
@@ -79,9 +97,12 @@ int main(int argc, char *argv[])
 				op = OP_INDEX;
 			} else if (!strcmp(params[i+1], "vindex")) {
 				op = OP_VINDEX;
+			} else if (!strcmp(params[i+1], "photo")) {
+				op = OP_PHOTO;
 			}
 		} else if (!strcmp(params[i], "search")) {
 			search = params[i+1];
+			params[i+1] = NULL;
 			if (search != NULL) {
 				keyid = strtoul(search, &end, 16);
 				if (*search != 0 &&
@@ -90,6 +111,8 @@ int main(int argc, char *argv[])
 					ishex = true;
 				}
 			}
+		} else if (!strcmp(params[i], "idx")) {
+			indx = atoi(params[i+1]);
 		} else if (!strcmp(params[i], "fingerprint")) {
 			if (!strcmp(params[i+1], "on")) {
 				fingerprint = true;
@@ -98,51 +121,106 @@ int main(int argc, char *argv[])
 			if (!strcmp(params[i+1], "on")) {
 				exact = true;
 			}
+		} else if (!strcmp(params[i], "options")) {
+			/*
+			 * TODO: We should be smarter about this; options may
+			 * have several entries. For now mr is the only valid
+			 * one though.
+			 */
+			if (!strcmp(params[i+1], "mr")) {
+				mrhkp = true;
+			}
+		}
+		free(params[i]);
+		params[i] = NULL;
+		if (params[i+1] != NULL) {
+			free(params[i+1]);
+			params[i+1] = NULL;
 		}
 	}
+	if (params != NULL) {
+		free(params);
+		params = NULL;
+	}
 
-//	puts("HTTP/1.0 200 OK");
-//	puts("Server: onak 0.0.1");
-	puts("Content-Type: text/html\n");
-	puts("<html>\n<title>Lookup of key</title>");
-	puts("<body>");
+	if (mrhkp) {
+		puts("Content-Type: text/plain\n");
+	} else if (op == OP_PHOTO) {
+		puts("Content-Type: image/jpeg\n");
+	} else {
+		start_html("Lookup of key");
+	}
 
 	if (op == OP_UNKNOWN) {
 		puts("Error: No operation supplied.");
 	} else if (search == NULL) {
 		puts("Error: No key to search for supplied.");
 	} else {
-		initdb();
+		readconfig(NULL);
+		initlogthing("lookup", config.logfile);
+		catchsignals();
+		initdb(true);
 		switch (op) {
 		case OP_GET:
-			if (fetch_key(keyid, &publickey)) {
+			logthing(LOGTHING_NOTICE, "Getting keyid 0x%llX",
+					keyid);
+			if (fetch_key(keyid, &publickey, false)) {
 				puts("<pre>");
+				cleankeys(publickey);
 				flatten_publickey(publickey,
 							&packets,
 							&list_end);
-				armor_openpgp_stream(putnextchar,
+				armor_openpgp_stream(stdout_putchar,
 						NULL,
 						packets);
 				puts("</pre>");
 			} else {
+				logthing(LOGTHING_NOTICE,
+					"Failed to fetch key.");
 				puts("Key not found");
 			}
 			break;
 		case OP_INDEX:
 			find_keys(search, keyid, ishex, fingerprint, exact,
-					false);
+					false, mrhkp);
 			break;
 		case OP_VINDEX:
 			find_keys(search, keyid, ishex, fingerprint, exact,
-					true);
+					true, mrhkp);
+			break;
+		case OP_PHOTO:
+			if (fetch_key(keyid, &publickey, false)) {
+				unsigned char *photo = NULL;
+				size_t         length = 0;
+
+				if (getphoto(publickey, 0, &photo, &length)) {
+					fwrite(photo,
+							1,
+							length,
+							stdout);
+				}
+				free_publickey(publickey);
+				publickey = NULL;
+			}
 			break;
 		default:
 			puts("Unknown operation!");
 		}
 		cleanupdb();
+		cleanuplogthing();
+		cleanupconfig();
 	}
-	puts("<hr>");
-	puts("Produced by onak " VERSION " by Jonathan McDowell");
-	puts("</body>\n</html>");
+	if (!mrhkp) {
+		puts("<hr>");
+		puts("Produced by onak " PACKAGE_VERSION 
+				" by Jonathan McDowell");
+		end_html();
+	}
+
+	if (search != NULL) {
+		free(search);
+		search = NULL;
+	}
+	
 	return (EXIT_SUCCESS);
 }
